@@ -1,6 +1,8 @@
 import os
 import sys
-from threading import Thread, Lock
+import time
+from multiprocessing import Process
+from threading import Lock
 
 PY2 = sys.version_info[0] == 2
 if PY2:
@@ -11,7 +13,7 @@ else:
 TCPServer.allow_reuse_address = True
 
 HOST = "127.0.0.1"
-PORT_START = 10000
+PORT_START = sys.version_info[0] * 10000 + sys.version_info[1] * 100
 
 current_port = PORT_START
 test_lock = Lock()
@@ -41,7 +43,12 @@ def new_server_thread(handle_cls, port=None):
         (HOST, port),
         handle_cls
     )
+    p = Process(target=server_thread, args=(server,))
+    p.start()
+    # Give the server some time to bind
+    # Is there a better option?
+    time.sleep(0.2)
     return (
         "http://%s:%d" % (HOST, port),
-        Thread(target=server_thread, args=(server,))
-    )
\ No newline at end of file
+        p
+    )
